package net.asukaze.cassava.core.command;

import java.util.ArrayList;
import java.util.List;

public class CommandStack {
	private List<Command> undoStack = new ArrayList<Command>();
	private List<Command> redoStack = null;

	public void run(Command cmd) throws CommandException {
		if (cmd.run() == Command.CHANGED) {
			redoStack = null;
			if (cmd.canUndo()) {
				undoStack.add(cmd);
			} else {
				undoStack.clear();
			}
		}
	}

	public boolean canUndo() {
		int size = undoStack.size();
		return (size > 0 && undoStack.get(size - 1).canUndo());
	}

	public void undo() throws CommandException {
		if (canUndo()) {
			int last = undoStack.size() - 1;
			Command cmd = undoStack.get(last);
			if (cmd.undo() == Command.CHANGED) {
				if (redoStack == null) {
					redoStack = new ArrayList<Command>();
				}
				redoStack.add(cmd);
				undoStack.remove(last);
			}
		}
	}

	public boolean canRedo() {
		if (redoStack == null) {
			return false;
		}
		int last = redoStack.size() - 1;
		return (last >= 0 && redoStack.get(last).canUndo());
	}

	public void redo() throws CommandException {
		if (canRedo()) {
			int last = redoStack.size() - 1;
			Command cmd = redoStack.get(last);
			if (cmd.run() == Command.CHANGED) {
				if (cmd.canUndo()) {
					undoStack.add(cmd);
				} else {
					undoStack.clear();
				}
				redoStack.remove(last);
			}
		}
	}
}
