package net.asukaze.cassava.core.command;

import java.util.List;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public class DeleteColumnCommand implements Command {
	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private ICassavaTableModel deleted;
	private int leftColIndex;
	private int rightColIndex;

	public DeleteColumnCommand(ICassavaTableModel model,
			ICassavaViewModel viewModel, int leftColIndex, int rightColIndex) {
		this.model = model;
		this.viewModel = viewModel;
		this.deleted = null;
		this.leftColIndex = leftColIndex;
		this.rightColIndex = rightColIndex;
	}

	public boolean canUndo() {
		return true;
	}

	public Status run() throws CommandException {
		if (leftColIndex <= rightColIndex) {
			deleted = ModelUtil.copy(model, new Selection(leftColIndex, 0,
					rightColIndex, model.getRowCount() - 1));
			for (int x = rightColIndex; x >= leftColIndex; x--) {
				model.deleteColumn(x);
				List<Integer> colWidths = viewModel.getColWidths();
				if (colWidths.size() >= x) {
					colWidths.remove(x);
				}
			}
			int currentCol = viewModel.getCurrentCol();
			if (currentCol >= leftColIndex && currentCol <= rightColIndex) {
				currentCol = leftColIndex;
				viewModel.setCurrentCol(currentCol);
			} else if (currentCol > rightColIndex) {
				currentCol -= (rightColIndex - leftColIndex + 1);
				viewModel.setCurrentCol(currentCol);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

	public Status undo() throws CommandException {
		if (leftColIndex <= rightColIndex && deleted != null) {
			for (int x = leftColIndex; x <= rightColIndex; x++) {
				model.insertColumn(x);
			}
			ModelUtil.paste(model, new Selection(leftColIndex, 0,
					rightColIndex, model.getRowCount() - 1), deleted);
			deleted = null;
			int currentCol = viewModel.getCurrentCol();
			if (currentCol >= leftColIndex) {
				currentCol += (rightColIndex - leftColIndex + 1);
				viewModel.setCurrentCol(currentCol);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

}
