package net.asukaze.cassava.core.command;

import java.util.List;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;

public class InsertColumnCommand implements Command {
	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private int leftColIndex;
	private int rightColIndex;

	public InsertColumnCommand(ICassavaTableModel model,
			ICassavaViewModel viewModel, int leftColIndex, int rightColIndex) {
		this.model = model;
		this.viewModel = viewModel;
		this.leftColIndex = leftColIndex;
		this.rightColIndex = rightColIndex;
	}

	public boolean canUndo() {
		return true;
	}

	public Status run() throws CommandException {
		if (leftColIndex <= rightColIndex) {
			for (int x = leftColIndex; x <= rightColIndex; x++) {
				model.insertColumn(x);
				List<Integer> colWidths = viewModel.getColWidths();
				if (colWidths.size() >= x) {
					colWidths.add(x, viewModel.getDefaultColWidth());
				}
			}
			int currentCol = viewModel.getCurrentCol();
			if (currentCol >= leftColIndex) {
				currentCol += (rightColIndex - leftColIndex + 1);
				viewModel.setCurrentCol(currentCol);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

	public Status undo() throws CommandException {
		if (leftColIndex <= rightColIndex) {
			for (int x = rightColIndex; x >= leftColIndex; x--) {
				model.deleteColumn(x);
			}
			int currentCol = viewModel.getCurrentCol();
			if (currentCol >= leftColIndex && currentCol <= rightColIndex) {
				currentCol = leftColIndex;
				viewModel.setCurrentCol(currentCol);
			} else if (currentCol > rightColIndex) {
				currentCol -= (rightColIndex - leftColIndex + 1);
				viewModel.setCurrentCol(currentCol);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

}
