package net.asukaze.cassava.core.command;

import java.util.List;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;

public class InsertRowCommand implements Command {
	private ICassavaTableModel model;
	private ICassavaViewModel viewModel;
	private int topRowIndex;
	private int bottomRowIndex;

	public InsertRowCommand(ICassavaTableModel model,
			ICassavaViewModel viewModel, int topRowIndex, int bottomRowIndex) {
		this.model = model;
		this.viewModel = viewModel;
		this.topRowIndex = topRowIndex;
		this.bottomRowIndex = bottomRowIndex;
	}

	public boolean canUndo() {
		return true;
	}

	public Status run() {
		if (topRowIndex <= bottomRowIndex) {
			for (int y = topRowIndex; y <= bottomRowIndex; y++) {
				model.insertRow(y);
				List<Integer> rowHeights = viewModel.getRowHeights();
				if (rowHeights.size() >= y) {
					rowHeights.add(y, viewModel.getDefaultRowHeight());
				}
			}
			int currentRow = viewModel.getCurrentRow();
			if (currentRow >= topRowIndex) {
				currentRow += (bottomRowIndex - topRowIndex + 1);
				viewModel.setCurrentRow(currentRow);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}

	public Status undo() {
		if (topRowIndex <= bottomRowIndex) {
			for (int y = bottomRowIndex; y >= topRowIndex; y--) {
				model.deleteRow(y);
			}
			int currentRow = viewModel.getCurrentRow();
			if (currentRow >= topRowIndex && currentRow <= bottomRowIndex) {
				currentRow = topRowIndex;
				viewModel.setCurrentRow(currentRow);
			} else if (currentRow > bottomRowIndex) {
				currentRow -= (bottomRowIndex - topRowIndex + 1);
				viewModel.setCurrentRow(currentRow);
			}
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}
}
