package net.asukaze.cassava.core.model;

import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.List;

import javax.swing.table.AbstractTableModel;

/**
 * e[uf[^. 0 IW.
 */
@SuppressWarnings("serial")
public class CassavaTableModel extends AbstractTableModel implements
		ICassavaTableModel {
	private static final String EMPTY = ""; //$NON-NLS-1$

	private int colCount = 0;
	private List<List<String>> data = new ArrayList<List<String>>();
	private boolean modified = false;
	private String filename = null;
	private String lineSeparator = System.getProperty("line.separator"); //$NON-NLS-1$
	private Charset charset = Charset.defaultCharset();
	private DataFormat dataFormat = null;
	private CellCalculator cellCalculator = null;

	@Override
	public Class<?> getColumnClass(int columnIndex) {
		return String.class;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getColumnCount()
	 */
	public int getColumnCount() {
		return colCount;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#getColumnCount(int)
	 */
	public int getColumnCount(int rowIndex) {
		if (rowIndex < 0 || rowIndex >= data.size()) {
			return 0;
		}
		List<String> line = data.get(rowIndex);
		return line.size();
	}

	@Override
	public String getColumnName(int columnIndex) {
		return Integer.toString(columnIndex + 1);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getRowCount()
	 */
	public int getRowCount() {
		return data.size();
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getValueAt(int,
	 * int)
	 */
	public String getValueAt(int rowIndex, int columnIndex) {
		if (rowIndex < 0 || columnIndex < 0 || columnIndex >= colCount
				|| rowIndex >= data.size()) {
			return EMPTY;
		}
		List<String> line = data.get(rowIndex);
		if (columnIndex >= line.size()) {
			return EMPTY;
		}
		return line.get(columnIndex);
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#getCalculatedValueAt
	 * (int, int)
	 */
	public String getCalculatedValueAt(int rowIndex, int columnIndex) {
		String raw = getValueAt(rowIndex, columnIndex);
		try {
			if (cellCalculator != null) {
				return cellCalculator.calc(raw, rowIndex, columnIndex);
			} else {
				return raw;
			}
		} catch (Exception ex) {
			// TODO: G[Ԃ
			return raw;
		}
	}

	@Override
	public boolean isCellEditable(int rowIndex, int columnIndex) {
		return true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setValueAt(java.lang
	 * .Object, int, int)
	 */
	@Override
	public void setValueAt(Object value, int rowIndex, int columnIndex) {
		if (rowIndex < 0 || columnIndex < 0) {
			return;
		}
		if (columnIndex >= colCount) {
			colCount = columnIndex + 1;
			fireTableStructureChanged();
		}
		if (rowIndex >= data.size()) {
			int firstRow = data.size();
			while (rowIndex >= data.size()) {
				data.add(new ArrayList<String>());
			}
			fireTableRowsInserted(firstRow, rowIndex);
		}
		List<String> line = data.get(rowIndex);
		while (columnIndex >= line.size()) {
			line.add(EMPTY);
		}

		line.set(columnIndex, value.toString());

		fireTableCellUpdated(rowIndex, columnIndex);
		modified = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setModified(boolean)
	 */
	public void setModified(boolean modified) {
		this.modified = modified;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#isModified()
	 */
	public boolean isModified() {
		return modified;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setFilename(java.lang
	 * .String)
	 */
	public void setFilename(String filename) {
		this.filename = filename;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getFilename()
	 */
	public String getFilename() {
		return filename;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setLineSeparator(java
	 * .lang.String)
	 */
	public void setLineSeparator(String lineSeparator) {
		this.lineSeparator = lineSeparator;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getLineSeparator()
	 */
	public String getLineSeparator() {
		return lineSeparator;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setCharset(java.nio
	 * .charset.Charset)
	 */
	public void setCharset(Charset charset) {
		this.charset = charset;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#getCharset()
	 */
	public Charset getCharset() {
		return charset;
	}

	public void setDataFormat(DataFormat dataFormat) {
		this.dataFormat = dataFormat;
	}

	public DataFormat getDataFormat() {
		if (dataFormat == null) {
			return DataFormat.RFC4180;
		} else {
			return dataFormat;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setCellCalculator(net
	 * .asukaze.cassava.core.model.CellCalculator)
	 */
	public void setCellCalculator(CellCalculator cellCalculator) {
		this.cellCalculator = cellCalculator;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#getCellCalculator()
	 */
	public CellCalculator getCellCalculator() {
		return cellCalculator;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#insertRow(int)
	 */
	public void insertRow(int rowIndex) {
		if (rowIndex >= data.size()) {
			data.add(new ArrayList<String>());
		} else {
			data.add(rowIndex, new ArrayList<String>());
		}
		fireTableRowsInserted(rowIndex, rowIndex);
		modified = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#deleteRow(int)
	 */
	public void deleteRow(int rowIndex) {
		if (rowIndex < data.size()) {
			data.remove(rowIndex);
			fireTableRowsDeleted(rowIndex, rowIndex);
			modified = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#setRowCount(int)
	 */
	public void setRowCount(int rowCount) {
		if (rowCount == data.size()) {
			return;
		} else if (rowCount < data.size()) {
			int lastRow = data.size();
			data.subList(rowCount, data.size()).clear();
			fireTableRowsDeleted(rowCount, lastRow);
		} else {
			int firstRow = data.size();
			while (rowCount >= data.size()) {
				data.add(new ArrayList<String>());
			}
			fireTableRowsInserted(firstRow, rowCount);
		}
		modified = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#insertColumn(int)
	 */
	public void insertColumn(int columnIndex) {
		for (List<String> row : data) {
			if (columnIndex >= row.size()) {
				row.add(EMPTY);
			} else {
				row.add(columnIndex, EMPTY);
			}
		}
		colCount++;
		fireTableStructureChanged();
		modified = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#deleteColumn(int)
	 */
	public void deleteColumn(int columnIndex) {
		int maxColCount = 0;
		for (List<String> row : data) {
			if (columnIndex < row.size()) {
				row.remove(columnIndex);
			}
			maxColCount = Math.max(maxColCount, row.size());
		}
		if (maxColCount != colCount) {
			colCount = maxColCount;
			fireTableStructureChanged();
			modified = true;
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * net.asukaze.cassava.core.model.ICassavaTableModel#setColumnCount(int)
	 */
	public void setColumnCount(int columnCount) {
		if (columnCount == colCount) {
			return;
		}
		for (List<String> row : data) {
			if (row.size() >= columnCount) {
				row.subList(columnCount, row.size()).clear();
			}
		}
		colCount = columnCount;
		fireTableStructureChanged();
		modified = true;
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see net.asukaze.cassava.core.model.ICassavaTableModel#clear()
	 */
	public void clear() {
		colCount = 0;
		data = new ArrayList<List<String>>();
		fireTableStructureChanged();
		modified = false;
	}

	@Override
	public void insertCellsRight(int rowIndex, int columnIndex, int insertRows,
			int insertColumns) {
		List<String> insertData = new ArrayList<String>(insertColumns);
		for (int i = 0; i < insertColumns; i++) {
			insertData.add(EMPTY);
		}
		int maxColCount = colCount;
		for (int y = rowIndex; y < rowIndex + insertRows; y++) {
			if (y >= data.size()) {
				break;
			}
			List<String> row = data.get(y);
			if (row.size() > columnIndex) {
				row.addAll(columnIndex, insertData);
				maxColCount = Math.max(maxColCount, row.size());
			}
		}
		if (maxColCount > colCount) {
			colCount = maxColCount;
			fireTableStructureChanged();
		}
		modified = true;
	}

	@Override
	public void deleteCellsLeft(int rowIndex, int columnIndex, int deleteRows,
			int deleteColumns) {
		for (int y = rowIndex; y < rowIndex + deleteRows; y++) {
			if (y >= data.size()) {
				break;
			}
			List<String> row = data.get(y);
			if (row.size() > columnIndex) {
				int toIndex = Math.min(columnIndex + deleteColumns, row.size());
				row.subList(columnIndex, toIndex).clear();
			}
		}
	}
}
