package net.asukaze.cassava.core.model;

public class DataFormat {
	private static final String LINE_SEPARATORS = "\r\n"; //$NON-NLS-1$

	private static final String RFC4180_STRONG_DELIMITERS = ","; //$NON-NLS-1$
	private static final String RFC4180_WEAK_DELIMITERS = ""; //$NON-NLS-1$
	private static final String RFC4180_QUOTE_CHARS = "\""; //$NON-NLS-1$
	private static final boolean RFC4180_USE_QUOTE = true;
	private static final boolean RFC4180_OMIT_COMMA = false;

	public static DataFormat RFC4180 = new DataFormat(null, false,
			new String[0], RFC4180_STRONG_DELIMITERS, RFC4180_WEAK_DELIMITERS,
			RFC4180_QUOTE_CHARS, RFC4180_USE_QUOTE, RFC4180_OMIT_COMMA);

	private String name;
	private boolean isDefault;
	private String[] exts;
	private String strongDelimiters;
	private String weakDelimiters;
	private String quoteChars;
	private boolean useQuote;
	private boolean omitComma;

	public DataFormat(String name, boolean isDefault, String[] exts,
			String strongDelimiters, String weakDelimiters, String quoteChars,
			boolean useQuote, boolean omitComma) {
		this.name = name;
		this.isDefault = isDefault;
		this.exts = exts.clone();
		this.strongDelimiters = strongDelimiters;
		this.weakDelimiters = weakDelimiters;
		this.quoteChars = quoteChars;
		this.useQuote = useQuote;
		this.omitComma = omitComma;
	}

	public String getName() {
		return name;
	}

	public boolean isDefault() {
		return isDefault;
	}

	public String[] getExts() {
		return exts.clone();
	}

	public boolean isStrongDelimiter(char ch) {
		return (strongDelimiters.indexOf(ch) >= 0);
	}

	public char getDefaultDelimiter() {
		return strongDelimiters.charAt(0);
	}

	public boolean isWeakDelimiter(char ch) {
		return (weakDelimiters.indexOf(ch) >= 0);
	}

	public char[] getAllSpecialChars() {
		return (strongDelimiters + weakDelimiters + quoteChars + LINE_SEPARATORS)
				.toCharArray();
	}

	public boolean isUseQuote() {
		return useQuote;
	}

	public boolean isQuoteChar(char ch) {
		return (quoteChars.indexOf(ch) >= 0);
	}

	public char getDefaultQuoteChar() {
		return quoteChars.charAt(0);
	}

	public String getQuoteChars() {
		return quoteChars;
	}

	public boolean isOmitComma() {
		return omitComma;
	}
}
