package net.asukaze.cassava.io.command;

import java.io.CharArrayWriter;
import java.io.IOException;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;
import net.asukaze.cassava.io.util.ClipboardUtil;
import net.asukaze.cassava.io.util.CsvWriter;

public class CopyCommand implements Command {
	public enum Type {
		COPY, CUT
	}

	private static final String LINE_SEPARATOR = System
			.getProperty("line.separator"); //$NON-NLS-1$

	private ICassavaTableModel model;
	private Selection selection;
	private DataFormat format;
	private Type type;
	private ICassavaTableModel deleted;

	public CopyCommand(ICassavaTableModel model, Selection selection,
			DataFormat format, Type type) {
		this.model = model;
		this.selection = selection;
		this.format = format;
		this.type = type;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public Status run() throws CommandException {
		try {
			// Nbv{[hփRs[
			CharArrayWriter caw = new CharArrayWriter();
			CsvWriter csvw = new CsvWriter(format, caw, LINE_SEPARATOR);
			try {
				csvw.writeTable(model, selection);
			} finally {
				csvw.close();
				caw.close();
			}
			ClipboardUtil.copy(caw.toString());

			// ؂Ȃf[^폜
			if (type == Type.CUT) {
				deleted = ModelUtil.cut(model, selection);
				return Status.CHANGED;
			} else {
				deleted = null;
				return Status.NO_CHANGE;
			}
		} catch (IOException ex) {
			throw new CommandException(ex);
		}
	}

	@Override
	public Status undo() throws CommandException {
		if (deleted != null) {
			ModelUtil.paste(model, selection, deleted);
			deleted = null;
			return Status.CHANGED;
		} else {
			return Status.NO_CHANGE;
		}
	}
}
