package net.asukaze.cassava.io.command;

import java.io.IOException;
import java.io.Writer;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.io.util.CsvWriter;

public class SaveFileCommand implements Command {
	private static final String LINE_SEPARATOR = System
			.getProperty("line.separator"); //$NON-NLS-1$

	private ICassavaTableModel model;
	private Writer writer;
	private String filename;

	public SaveFileCommand(ICassavaTableModel model, Writer writer, String filename) {
		this.model = model;
		this.writer = writer;
		this.filename = filename;
	}

	public boolean canUndo() {
		return false;
	}

	public Status run() throws CommandException {
		CsvWriter csvw = new CsvWriter(model.getDataFormat(), writer, LINE_SEPARATOR);
		try {
			csvw.writeTable(model);
			if (filename != null) {
				model.setFilename(filename);
			}
		} catch (IOException ex) {
			throw new CommandException(ex);
		} finally {
			try {
				csvw.close();
				this.model = null;
				this.writer = null;
				this.filename = null;
			} catch (IOException ex) {
				throw new CommandException(ex);
			}
		}
		return Status.NO_CHANGE;
	}

	public Status undo() throws CommandException {
		return Status.FAIL;
	}
}
