package net.asukaze.cassava.io.util;

import java.io.BufferedWriter;
import java.io.IOException;
import java.io.Writer;

import net.asukaze.cassava.core.model.DataFormat;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;

public class CsvWriter {
	private DataFormat dataFormat;
	private BufferedWriter writer;
	private String lineSeparator;

	public CsvWriter(DataFormat dataFormat, Writer writer, String lineSeparator) {
		this.dataFormat = dataFormat;
		this.writer = new BufferedWriter(writer);
		this.lineSeparator = lineSeparator;
	}

	public void flush() throws IOException {
		writer.flush();
	}

	public void close() throws IOException {
		writer.close();
	}

	@Override
	protected void finalize() throws IOException {
		close();
	}

	private void writeCell(String value) throws IOException {
		boolean quote = false;
		if (dataFormat.isUseQuote()) {
			for (char ch : dataFormat.getAllSpecialChars()) {
				if (value.indexOf(ch) >= 0) {
					quote = true;
					break;
				}
			}
		}
		if (quote) {
			writer.append(dataFormat.getDefaultQuoteChar());
			for (char ch : dataFormat.getQuoteChars().toCharArray()) {
				String q = String.valueOf(ch);
				value = value.replace(q, q + q);
			}
			writer.write(value);
			writer.append(dataFormat.getDefaultQuoteChar());
		} else {
			writer.write(value);
		}
	}

	public void writeTable(ICassavaTableModel model, Selection selection)
			throws IOException {
		for (int y = selection.top; y <= selection.bottom; y++) {
			for (int x = selection.left; x <= selection.right; x++) {
				if (x > selection.left) {
					writer.append(dataFormat.getDefaultDelimiter());
				}
				writeCell(model.getValueAt(y, x));
			}
			writer.write(lineSeparator);
		}
	}

	public void writeTable(ICassavaTableModel model) throws IOException {
		int rowCount = model.getRowCount();
		int columnCount = model.getColumnCount();
		writeTable(model, new Selection(0, 0, columnCount-1, rowCount-1));
	}
}
