package net.asukaze.cassava.core.util;

import static org.junit.Assert.*;

import net.asukaze.cassava.core.model.CassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;

import org.junit.Before;
import org.junit.Test;

public class ModelUtilTest {
	ICassavaTableModel model;

	@Before
	public void setUp() {
		model = new CassavaTableModel();
		model.setValueAt("AAA", 0, 0); //$NON-NLS-1$
		model.setValueAt("BBB", 0, 1); //$NON-NLS-1$
		model.setValueAt("CCC", 1, 0); //$NON-NLS-1$
		model.setValueAt("DDD", 1, 1); //$NON-NLS-1$
		model.setValueAt("EEE", 2, 0); //$NON-NLS-1$
		model.setValueAt("FFF", 2, 1); //$NON-NLS-1$
		model.setValueAt("GGG", 3, 0); //$NON-NLS-1$
		model.setValueAt("HHH", 3, 1); //$NON-NLS-1$
	}

	@Test
	public void testCut() {
		ICassavaTableModel r = ModelUtil.cut(model, new Selection(0, 1, 1, 2));
		assertEquals(2, r.getRowCount());
		assertEquals(2, r.getColumnCount());
		assertEquals("CCC", r.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("DDD", r.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("EEE", r.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("FFF", r.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("AAA", model.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("BBB", model.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("", model.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("", model.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("", model.getValueAt(2, 0)); //$NON-NLS-1$
		assertEquals("", model.getValueAt(2, 1)); //$NON-NLS-1$
		assertEquals("GGG", model.getValueAt(3, 0)); //$NON-NLS-1$
		assertEquals("HHH", model.getValueAt(3, 1)); //$NON-NLS-1$
	}

	@Test
	public void testCopy() {
		ICassavaTableModel r = ModelUtil.copy(model, new Selection(0, 1, 1, 2));
		assertEquals(2, r.getRowCount());
		assertEquals(2, r.getColumnCount());
		assertEquals("CCC", r.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("DDD", r.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("EEE", r.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("FFF", r.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("AAA", model.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("BBB", model.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("CCC", model.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("DDD", model.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("EEE", model.getValueAt(2, 0)); //$NON-NLS-1$
		assertEquals("FFF", model.getValueAt(2, 1)); //$NON-NLS-1$
		assertEquals("GGG", model.getValueAt(3, 0)); //$NON-NLS-1$
		assertEquals("HHH", model.getValueAt(3, 1)); //$NON-NLS-1$
	}

	@Test
	public void testPaste() {
		ICassavaTableModel data = new CassavaTableModel();
		data.setValueAt("<1>", 0, 0); //$NON-NLS-1$
		data.setValueAt("<2>", 0, 1); //$NON-NLS-1$
		data.setValueAt("<3>", 1, 0); //$NON-NLS-1$
		data.setValueAt("<4>", 1, 1); //$NON-NLS-1$
		ICassavaTableModel r = ModelUtil.paste(model, new Selection(0, 1, 1, 2),
				data);
		assertEquals(2, r.getRowCount());
		assertEquals(2, r.getColumnCount());
		assertEquals("CCC", r.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("DDD", r.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("EEE", r.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("FFF", r.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("AAA", model.getValueAt(0, 0)); //$NON-NLS-1$
		assertEquals("BBB", model.getValueAt(0, 1)); //$NON-NLS-1$
		assertEquals("<1>", model.getValueAt(1, 0)); //$NON-NLS-1$
		assertEquals("<2>", model.getValueAt(1, 1)); //$NON-NLS-1$
		assertEquals("<3>", model.getValueAt(2, 0)); //$NON-NLS-1$
		assertEquals("<4>", model.getValueAt(2, 1)); //$NON-NLS-1$
		assertEquals("GGG", model.getValueAt(3, 0)); //$NON-NLS-1$
		assertEquals("HHH", model.getValueAt(3, 1)); //$NON-NLS-1$
	}
}
