package net.asukaze.cassava.ui.action;

import java.awt.Frame;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JMenuItem;
import javax.swing.JTable;
import javax.swing.table.TableCellEditor;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.ui.Log;
import net.asukaze.cassava.ui.IApplication;
import net.asukaze.cassava.ui.table.CassavaGrid;

public abstract class Action implements ActionListener {
	private IApplication application = null;
	private boolean hideEditor = false;

	public void setApplication(IApplication application) {
		this.application = application;
	}

	protected void setHideEditor(boolean hideEditor) {
		this.hideEditor = hideEditor;
	}

	protected static void stopCellEditing(JTable table) {
		TableCellEditor editor = table.getCellEditor();
		if (editor != null) {
			editor.stopCellEditing();
		}
	}

	/**
	 * Called when menu clicked.
	 */
	public void actionPerformed(ActionEvent event) {
		try {
			CassavaGrid grid = getCassavaGrid();
			if (hideEditor) {
				stopCellEditing(grid);
			}
			Command cmd = run();
			if (cmd != null) {
				grid.getCommandStack().run(cmd);
			}
			afterFirstRun();
			grid.updateUI();
		} catch (Exception ex) {
			Log.error(ex);
		}
	}

	protected IApplication getApplication() {
		return application;
	}

	protected CassavaGrid getCassavaGrid() {
		return application.getCassavaGrid();
	}

	protected Frame getFrame() {
		return application.getFrame();
	}

	protected ICassavaTableModel getTableModel() {
		return application.getCassavaGrid().getTableModel();
	}

	protected ICassavaViewModel getViewModel() {
		return application.getCassavaGrid().getViewModel();
	}

	public JMenuItem createMenuItem() {
		return new JMenuItem();
	}

	/**
	 * Execute Action
	 * 
	 * @return command to execute or null
	 * @throws Exception
	 */
	public abstract Command run() throws Exception;

	/**
	 * Command sɌĂяo.<br>
	 * s GUI ̒ōs.<br>
	 * AhDEhDɂ͌ĂяoȂ.<br>
	 */
	public void afterFirstRun() {
	}
}
