package net.asukaze.cassava.ui.command;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public class ConnectRowCommand implements Command {
	private ICassavaTableModel model;
	private int firstRow;
	private int secondRow;
	private int columnCount;
	private int firstColumn;

	public ConnectRowCommand(ICassavaTableModel model, int firstRow,
			int secondRow) {
		this.model = model;
		this.firstRow = firstRow;
		this.secondRow = secondRow;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	private int getRowColumnCount(int rowIndex) {
		int count = model.getColumnCount(rowIndex) - 1;
		while (count >= 0 && model.getValueAt(rowIndex, count).isEmpty()) {
			count--;
		}
		return count + 1;
	}

	@Override
	public Status run() throws CommandException {
		columnCount = model.getColumnCount();
		firstColumn = getRowColumnCount(firstRow);
		for (int x = 0; x < columnCount; x++) {
			String value = model.getValueAt(secondRow, x);
			if (value.length() > 0) {
				model.setValueAt(value, firstRow, firstColumn + x);
			}
		}
		model.deleteRow(secondRow);
		return CHANGED;
	}

	@Override
	public Status undo() throws CommandException {
		model.insertRow(secondRow);
		ICassavaTableModel right = ModelUtil.cut(model, new Selection(
				firstColumn, firstRow, model.getColumnCount() - 1, firstRow));
		ModelUtil.paste(model, new Selection(0, secondRow, right
				.getColumnCount() - 1, secondRow), right);
		model.setColumnCount(columnCount);
		return CHANGED;
	}
}
