package net.asukaze.cassava.ui.command;

import net.asukaze.cassava.core.command.Command;
import net.asukaze.cassava.core.command.CommandException;
import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.Selection;
import net.asukaze.cassava.core.util.ModelUtil;

public class SplitRowCommand implements Command {
	private ICassavaTableModel model;
	private int rowIndex;
	private int columnIndex;
	private int selStart;

	public SplitRowCommand(ICassavaTableModel model, int rowIndex,
			int columnIndex, int selStart) {
		this.model = model;
		this.rowIndex = rowIndex;
		this.columnIndex = columnIndex;
		this.selStart = selStart;
	}

	@Override
	public boolean canUndo() {
		return true;
	}

	@Override
	public Status run() throws CommandException {
		model.insertRow(rowIndex + 1);
		ICassavaTableModel right = ModelUtil.cut(model,
				new Selection(columnIndex + 1, rowIndex,
						model.getColumnCount() - 1, rowIndex));
		ModelUtil.paste(model, new Selection(1, rowIndex + 1, right
				.getColumnCount(), rowIndex + 1), right);
		String cell = model.getValueAt(rowIndex, columnIndex);
		model.setValueAt(cell.substring(0, selStart), rowIndex, columnIndex);
		model.setValueAt(cell.substring(selStart), rowIndex + 1, 0);
		return CHANGED;
	}

	@Override
	public Status undo() throws CommandException {
		String last = model.getValueAt(rowIndex, columnIndex);
		String first = model.getValueAt(rowIndex + 1, 0);
		model.setValueAt(last + first, rowIndex, columnIndex);
		int cols = model.getColumnCount() - columnIndex - 1;
		ICassavaTableModel right = ModelUtil.copy(model, new Selection(1,
				rowIndex + 1, cols, rowIndex + 1));
		ModelUtil.paste(model, new Selection(columnIndex + 1, rowIndex, model
				.getColumnCount() - 1, rowIndex), right);
		model.deleteRow(rowIndex + 1);
		return CHANGED;
	}
}
