/*
 * SortDialog.java
 *
 * Created on 2009/03/14, 14:52:48
 */
package net.asukaze.cassava.ui.dialog;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.model.Selection;

@SuppressWarnings("serial")
public class SortDialog extends javax.swing.JDialog {

    private static boolean DEFAULT_ORDER_ASC = true;
    private static boolean DEFAULT_NUMSORT = true;
    private SortListener listener;
    private ICassavaViewModel viewModel;

    /** Creates new form SortDialog */
    public SortDialog(java.awt.Frame parent, boolean modal,
            ICassavaTableModel tableModel, ICassavaViewModel viewModel,
            SortListener listener) {
        super(parent, modal);
        initComponents();
        this.listener = listener;
        this.viewModel = viewModel;
        Selection selection = viewModel.getSelection();
        if (selection.left == selection.right) {
            leftSpinner.setValue(1);
            rightSpinner.setValue(tableModel.getColumnCount());
        } else {
            leftSpinner.setValue(selection.left + 1);
            rightSpinner.setValue(selection.right + 1);
        }
        topSpinner.setValue(selection.top + 1);
        if (selection.top == selection.bottom) {
            bottomSpinner.setValue(tableModel.getRowCount());
        } else {
            bottomSpinner.setValue(selection.bottom + 1);
        }
        keySpinner.setValue(viewModel.getCurrentCol() + 1);
        if (DEFAULT_ORDER_ASC) {
            rbOrderAsc.setSelected(true);
        } else {
            rbOrderDesc.setSelected(true);
        }
        cbNumSort.setSelected(DEFAULT_NUMSORT);
        this.setLocationRelativeTo(parent);
    }

    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        bgOrder = new javax.swing.ButtonGroup();
        tabbedPane = new javax.swing.JTabbedPane();
        sortOptionPanel = new javax.swing.JPanel();
        labelRange = new javax.swing.JLabel();
        leftSpinner = new javax.swing.JSpinner();
        topSpinner = new javax.swing.JSpinner();
        labelX = new javax.swing.JLabel();
        rightSpinner = new javax.swing.JSpinner();
        bottomSpinner = new javax.swing.JSpinner();
        labelKey = new javax.swing.JLabel();
        keySpinner = new javax.swing.JSpinner();
        labelOrder = new javax.swing.JLabel();
        rbOrderAsc = new javax.swing.JRadioButton();
        rbOrderDesc = new javax.swing.JRadioButton();
        cbNumSort = new javax.swing.JCheckBox();
        buttonPanel = new javax.swing.JPanel();
        okButton = new javax.swing.JButton();
        cancelButton = new javax.swing.JButton();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        java.util.ResourceBundle bundle = java.util.ResourceBundle.getBundle("net/asukaze/cassava/ui/messages"); // NOI18N
        setTitle(bundle.getString("SORT_DIALOG_TITLE")); // NOI18N

        tabbedPane.setPreferredSize(new java.awt.Dimension(280, 140));

        labelRange.setText(bundle.getString("SORT_DIALOG_RANGE")); // NOI18N

        leftSpinner.setMinimumSize(new java.awt.Dimension(40, 20));
        leftSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        leftSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                SortDialog.this.stateChanged(evt);
            }
        });

        topSpinner.setMinimumSize(new java.awt.Dimension(40, 20));
        topSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        topSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                SortDialog.this.stateChanged(evt);
            }
        });

        labelX.setText("-");

        rightSpinner.setMinimumSize(new java.awt.Dimension(40, 20));
        rightSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        rightSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                SortDialog.this.stateChanged(evt);
            }
        });

        bottomSpinner.setMinimumSize(new java.awt.Dimension(40, 20));
        bottomSpinner.setPreferredSize(new java.awt.Dimension(40, 20));
        bottomSpinner.addChangeListener(new javax.swing.event.ChangeListener() {
            public void stateChanged(javax.swing.event.ChangeEvent evt) {
                SortDialog.this.stateChanged(evt);
            }
        });

        labelKey.setText(bundle.getString("SORT_DIALOG_KEY")); // NOI18N

        keySpinner.setMinimumSize(new java.awt.Dimension(40, 20));
        keySpinner.setPreferredSize(new java.awt.Dimension(40, 20));

        labelOrder.setText(bundle.getString("SORT_DIALOG_ORDER")); // NOI18N

        bgOrder.add(rbOrderAsc);
        rbOrderAsc.setText(bundle.getString("SORT_DIALOG_ASC")); // NOI18N

        bgOrder.add(rbOrderDesc);
        rbOrderDesc.setText(bundle.getString("SORT_DIALOG_DESC")); // NOI18N

        cbNumSort.setText(bundle.getString("SORT_DIALOG_NUMBER")); // NOI18N

        javax.swing.GroupLayout sortOptionPanelLayout = new javax.swing.GroupLayout(sortOptionPanel);
        sortOptionPanel.setLayout(sortOptionPanelLayout);
        sortOptionPanelLayout.setHorizontalGroup(
            sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sortOptionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(sortOptionPanelLayout.createSequentialGroup()
                        .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.TRAILING)
                            .addComponent(labelOrder)
                            .addComponent(labelKey)
                            .addComponent(labelRange))
                        .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                        .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                            .addComponent(keySpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                            .addGroup(sortOptionPanelLayout.createSequentialGroup()
                                .addComponent(leftSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(topSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(labelX)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(rightSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(bottomSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                            .addGroup(sortOptionPanelLayout.createSequentialGroup()
                                .addComponent(rbOrderAsc)
                                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                                .addComponent(rbOrderDesc))))
                    .addComponent(cbNumSort))
                .addContainerGap(39, Short.MAX_VALUE))
        );
        sortOptionPanelLayout.setVerticalGroup(
            sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(sortOptionPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelRange)
                    .addComponent(leftSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(topSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(labelX)
                    .addComponent(rightSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE)
                    .addComponent(bottomSpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelKey)
                    .addComponent(keySpinner, javax.swing.GroupLayout.PREFERRED_SIZE, javax.swing.GroupLayout.DEFAULT_SIZE, javax.swing.GroupLayout.PREFERRED_SIZE))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addGroup(sortOptionPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.BASELINE)
                    .addComponent(labelOrder)
                    .addComponent(rbOrderAsc)
                    .addComponent(rbOrderDesc))
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cbNumSort)
                .addContainerGap(32, Short.MAX_VALUE))
        );

        tabbedPane.addTab(bundle.getString("SORT_DIALOG_TAB_SORT"), sortOptionPanel); // NOI18N

        getContentPane().add(tabbedPane, java.awt.BorderLayout.CENTER);

        buttonPanel.setPreferredSize(new java.awt.Dimension(120, 120));

        okButton.setText(bundle.getString("SORT_DIALOG_OK")); // NOI18N
        okButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                sort(evt);
            }
        });

        cancelButton.setText(bundle.getString("SORT_DIALOG_CANCEL")); // NOI18N
        cancelButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                cancel(evt);
            }
        });

        javax.swing.GroupLayout buttonPanelLayout = new javax.swing.GroupLayout(buttonPanel);
        buttonPanel.setLayout(buttonPanelLayout);
        buttonPanelLayout.setHorizontalGroup(
            buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonPanelLayout.createSequentialGroup()
                .addGroup(buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
                    .addGroup(buttonPanelLayout.createSequentialGroup()
                        .addGap(12, 12, 12)
                        .addComponent(okButton))
                    .addGroup(buttonPanelLayout.createSequentialGroup()
                        .addContainerGap()
                        .addComponent(cancelButton)))
                .addContainerGap(41, Short.MAX_VALUE))
        );
        buttonPanelLayout.setVerticalGroup(
            buttonPanelLayout.createParallelGroup(javax.swing.GroupLayout.Alignment.LEADING)
            .addGroup(buttonPanelLayout.createSequentialGroup()
                .addContainerGap()
                .addComponent(okButton)
                .addPreferredGap(javax.swing.LayoutStyle.ComponentPlacement.RELATED)
                .addComponent(cancelButton)
                .addContainerGap(101, Short.MAX_VALUE))
        );

        getContentPane().add(buttonPanel, java.awt.BorderLayout.EAST);

        pack();
    }// </editor-fold>//GEN-END:initComponents

    private int getValue(javax.swing.JSpinner spinner) {
        return ((Integer) spinner.getValue()).intValue();
    }

    private void sort(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_sort
        listener.sort(getValue(leftSpinner) - 1, getValue(topSpinner) - 1,
                getValue(rightSpinner) - 1, getValue(bottomSpinner) - 1,
                getValue(keySpinner) - 1, rbOrderDesc.isSelected(),
                cbNumSort.isSelected());
        close();
    }//GEN-LAST:event_sort

    private void cancel(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_cancel
        close();
    }//GEN-LAST:event_cancel

    private void stateChanged(javax.swing.event.ChangeEvent evt) {//GEN-FIRST:event_stateChanged
        if (this.isVisible() || evt == null) {
            viewModel.setSelection(new Selection(getValue(leftSpinner) - 1,
                    getValue(topSpinner) - 1, getValue(rightSpinner) - 1,
                    getValue(bottomSpinner) - 1));
        }
    }//GEN-LAST:event_stateChanged

    @Override
    public void setVisible(boolean b) {
        stateChanged(null);
        super.setVisible(b);
    }

    private void close() {
        DEFAULT_ORDER_ASC = rbOrderAsc.isSelected();
        DEFAULT_NUMSORT = cbNumSort.isSelected();
        this.setVisible(false);
        this.dispose();
    }
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup bgOrder;
    private javax.swing.JSpinner bottomSpinner;
    private javax.swing.JPanel buttonPanel;
    private javax.swing.JButton cancelButton;
    private javax.swing.JCheckBox cbNumSort;
    private javax.swing.JSpinner keySpinner;
    private javax.swing.JLabel labelKey;
    private javax.swing.JLabel labelOrder;
    private javax.swing.JLabel labelRange;
    private javax.swing.JLabel labelX;
    private javax.swing.JSpinner leftSpinner;
    private javax.swing.JButton okButton;
    private javax.swing.JRadioButton rbOrderAsc;
    private javax.swing.JRadioButton rbOrderDesc;
    private javax.swing.JSpinner rightSpinner;
    private javax.swing.JPanel sortOptionPanel;
    private javax.swing.JTabbedPane tabbedPane;
    private javax.swing.JSpinner topSpinner;
    // End of variables declaration//GEN-END:variables
}
