package net.asukaze.cassava.ui.table;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.List;

import javax.swing.JTable;
import javax.swing.table.TableCellEditor;
import javax.swing.table.TableColumn;
import javax.swing.table.TableColumnModel;

import net.asukaze.cassava.core.model.ICassavaViewModel;
import net.asukaze.cassava.core.model.Selection;

public class CassavaViewModel implements ICassavaViewModel {
	private JTable parent;
	private int defaultColWidth;
	private int defaultRowHeight;
	private int fixColCount = 0;
	private int fixRowCount = 0;

	public CassavaViewModel(JTable parent) {
		this.parent = parent;
	}

	@Override
	public int getColWidth(int colIndex) {
		try {
			return parent.getColumnModel().getColumn(colIndex).getWidth();
		} catch (NullPointerException ex) {
			return getDefaultColWidth();
		} catch (IndexOutOfBoundsException ex) {
			return getDefaultColWidth();
		}
	}

	@Override
	public List<Integer> getColWidths() {
		List<Integer> result = new ArrayList<Integer>();
		Enumeration<TableColumn> columns = parent.getColumnModel().getColumns();
		while (columns.hasMoreElements()) {
			TableColumn column = columns.nextElement();
			result.add(column.getWidth());
		}
		return result;
	}

	@Override
	public int getCurrentCol() {
		if (parent.isEditing()) {
			return parent.getEditingColumn();
		} else {
			return parent.getSelectedColumn();
		}
	}

	@Override
	public int getCurrentRow() {
		if (parent.isEditing()) {
			return parent.getEditingRow();
		} else {
			return parent.getSelectedRow();
		}
	}

	@Override
	public int getDefaultColWidth() {
		return defaultColWidth;
	}

	@Override
	public int getDefaultRowHeight() {
		return defaultRowHeight;
	}

	@Override
	public int getFixColCount() {
		return fixColCount;
	}

	@Override
	public int getFixRowCount() {
		return fixRowCount;
	}

	@Override
	public int getRowHeight(int rowIndex) {
		return parent.getRowHeight(rowIndex);
	}

	@Override
	public List<Integer> getRowHeights() {
		List<Integer> result = new ArrayList<Integer>();
		int rowCount = parent.getRowCount();
		for (int i = 0; i < rowCount; i++) {
			result.add(parent.getRowHeight(i));
		}
		return result;
	}

	@Override
	public Selection getSelection() {
		int left = parent.getSelectedColumn();
		int top = parent.getSelectedRow();
		int right = left + parent.getSelectedColumnCount() - 1;
		int bottom = top + parent.getSelectedRowCount() - 1;
		return new Selection(left, top, right, bottom);
	}

	@Override
	public void setColWidth(int colIndex, int colWidth) {
		parent.getColumnModel().getColumn(colIndex).setWidth(colWidth);
	}

	@Override
	public void setColWidths(List<Integer> colWidths) {
		TableColumnModel columnModel = parent.getColumnModel();
		for (int i = 0; i < colWidths.size(); i++) {
			columnModel.getColumn(i).setWidth(colWidths.get(i));
		}
	}

	private static void stopCellEditing(JTable table) {
		TableCellEditor editor = table.getCellEditor();
		if (editor != null) {
			editor.stopCellEditing();
		}
	}

	@Override
	public void setCurrentCol(int currentCol) {
		if (parent.isEditing()) {
			int row = parent.getEditingRow();
			stopCellEditing(parent);
			parent.editCellAt(row, currentCol);
		} else {
			parent.setColumnSelectionInterval(currentCol, currentCol);
		}
	}

	@Override
	public void setCurrentRow(int currentRow) {
		if (parent.isEditing()) {
			int column = parent.getEditingColumn();
			stopCellEditing(parent);
			parent.editCellAt(currentRow, column);
		} else {
			parent.setRowSelectionInterval(currentRow, currentRow);
		}
	}

	@Override
	public void setCurrentRowCol(int currentRow, int currentCol) {
		if (parent.isEditing()) {
			stopCellEditing(parent);
			parent.editCellAt(currentRow, currentCol);
		} else {
			parent.setRowSelectionInterval(currentRow, currentRow);
			parent.setColumnSelectionInterval(currentCol, currentCol);
		}
	}

	@Override
	public void setDefaultColWidth(int defaultColWidth) {
		this.defaultColWidth = defaultColWidth;
	}

	@Override
	public void setDefaultRowHeight(int defaultRowHeight) {
		this.defaultRowHeight = defaultRowHeight;
	}

	@Override
	public void setFixColCount(int fixColCount) {
		this.fixColCount = fixColCount;
	}

	@Override
	public void setFixRowCount(int fixRowCount) {
		this.fixRowCount = fixRowCount;
	}

	@Override
	public void setRowHeight(int rowIndex, int rowHeight) {
		parent.setRowHeight(rowIndex, rowHeight);
	}

	@Override
	public void setRowHeights(List<Integer> rowHeights) {
		for (int i = 0; i < rowHeights.size(); i++) {
			parent.setRowHeight(i, rowHeights.get(i));
		}
	}

	@Override
	public void setSelection(Selection selection) {
		if (parent.isEditing()) {
			stopCellEditing(parent);
		}
		parent.setRowSelectionInterval(selection.top, selection.bottom);
		parent.setColumnSelectionInterval(selection.left, selection.right);
	}
}
