package net.asukaze.cassava.macro.api;

import java.io.IOException;
import java.io.Writer;

import net.asukaze.cassava.core.model.ICassavaTableModel;
import net.asukaze.cassava.macro.APIFunctionHandler;
import net.asukaze.cassava.macro.FunctionNotFoundException;
import net.asukaze.cassava.macro.Macro;

public class ExportWriteFunctionHandler implements APIFunctionHandler {
	private Writer writer;

	public ExportWriteFunctionHandler(Writer writer) {
		this.writer = writer;
	}

	public Object runAPIFunction(String name, Object[] args, Macro macro)
			throws IOException {
		int n = args.length;
		if ("write".equals(name)) { //$NON-NLS-1$
			write(args, macro);
			return null;
		} else if ("writeln".equals(name)) { //$NON-NLS-1$
			writeln(args, macro);
			return null;
		}
		return new FunctionNotFoundException(name, n);
	}

	private void write(Object[] args, Macro macro) throws IOException {
		for (int i = 0; i < args.length; i++) {
			String text = macro.toString(args[i]);
			writer.write(text);
		}
	}

	private void writeln(Object[] args, Macro macro) throws IOException {
		write(args, macro);
		ICassavaTableModel model = macro.getEnvironment().getModel();
		if (model != null) {
			writer.write(model.getLineSeparator());
		} else {
			writer.write(System.getProperty("line.separator")); //$NON-NLS-1$
		}
	}

	public void close() throws IOException {
		writer.close();
	}
}
