package net.asukaze.cassava.macro;

import static org.junit.Assert.assertEquals;

import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.StringWriter;

import net.asukaze.cassava.macro.api.ExportWriteFunctionHandler;
import net.asukaze.cassava.macro.parser.ParseException;

import org.junit.Before;
import org.junit.Test;

public class StrIncTest {
	public static final String NL = "\r\n"; //$NON-NLS-1$

	private Macro macro;

	@Before
	public void setUp() {
		macro = new Macro();
	}

	public String run(String input) throws ParseException, IOException {
		StringWriter sw = new StringWriter();
		macro.registerAPIFunctionHandler(new ExportWriteFunctionHandler(sw));
		macro.run(new ByteArrayInputStream(input.getBytes()));
		return sw.toString();
	}

	public Object runExpr(String input) throws ParseException, IOException {
		return macro.runExpr(new ByteArrayInputStream(input.getBytes()));
	}

	@Test
	public void testEmpty() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"\")"); //$NON-NLS-1$
		assertEquals("1", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test0() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(0)"); //$NON-NLS-1$
		assertEquals("1", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test1() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(1)"); //$NON-NLS-1$
		assertEquals("2", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test2() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(2)"); //$NON-NLS-1$
		assertEquals("3", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test8() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(8)"); //$NON-NLS-1$
		assertEquals("9", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test9() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(9)"); //$NON-NLS-1$
		assertEquals("10", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test10() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(10)"); //$NON-NLS-1$
		assertEquals("11", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test19() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(19)"); //$NON-NLS-1$
		assertEquals("20", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test99() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(99)"); //$NON-NLS-1$
		assertEquals("100", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test999() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(999)"); //$NON-NLS-1$
		assertEquals("1000", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void test1099() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(1099)"); //$NON-NLS-1$
		assertEquals("1100", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA0() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A0\")"); //$NON-NLS-1$
		assertEquals("A1", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA9() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A9\")"); //$NON-NLS-1$
		assertEquals("A10", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA19() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A19\")"); //$NON-NLS-1$
		assertEquals("A20", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA99() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A99\")"); //$NON-NLS-1$
		assertEquals("A100", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA999() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A999\")"); //$NON-NLS-1$
		assertEquals("A1000", result.toString()); //$NON-NLS-1$
	}

	@Test
	public void testA1099() throws ParseException, IOException {
		Object result = runExpr("Lib.StrInc(\"A1099\")"); //$NON-NLS-1$
		assertEquals("A1100", result.toString()); //$NON-NLS-1$
	}
}
